var currentMsg = null;
var emgCount = 0;
var currentStatus = null;
var currentTabId = null;
var currentlySelectedMail = null;

browser.ecomailzapi.init();

function onMailCommand(command)
{
	if(command === MAILZ_CMD_SKIPDEADLINE)
	{
		setIcon(MAILZ_STATUS_LOADING);	
		EcoMailzRestService.skipDeadline(ecoMailzConnectionSettings, new EcoMailZMailDAO(currentlySelectedMail.id, currentlySelectedMail.archiveTimestamp), skipDeadlineSuccessCallback, skipDeadlineErrorCallback);
	}
	else if(command === MAILZ_CMD_MARK_DELETE)
	{
		setIcon(MAILZ_STATUS_LOADING);	
		EcoMailzRestService.markDeletion(ecoMailzConnectionSettings, new EcoMailZMailDAO(currentlySelectedMail.id, currentlySelectedMail.archiveTimestamp), markDeleteSuccessCallback, markDeleteErrorCallback);
	}
	else if(command === MAILZ_CMD_UNDO_DELETE)
	{
		setIcon(MAILZ_STATUS_LOADING);	
		EcoMailzRestService.undoDeletion(ecoMailzConnectionSettings, new EcoMailZMailDAO(currentlySelectedMail.id, currentlySelectedMail.archiveTimestamp), undoDeleteSuccessCallback, undoDeleteErrorCallback);
	}
}

function undoDeleteErrorCallback(errorMsg)
{
    setIcon(MAILZ_STATUS_TODELETE);
}

function markDeleteErrorCallback(errorMsg)
{
    setIcon(MAILZ_STATUS_TOARCHIVE);
}

function skipDeadlineErrorCallback(errorMsg)
{
    setIcon(MAILZ_STATUS_TOARCHIVE);
}

function undoDeleteSuccessCallback(response, type, xml, text)
{
    setIcon(MAILZ_STATUS_TOARCHIVE);
}

function markDeleteSuccessCallback(response, type, xml, text)
{
    setIcon(MAILZ_STATUS_TODELETE);
}

function skipDeadlineSuccessCallback(response, type, xml, text)
{
    setIcon(MAILZ_STATUS_ARCHIVED);
}

browser.messageDisplay.onMessageDisplayed.addListener((tab, message) => {
	//if(tab.mailTab)
	//{ 	
		emgCount = 0;
		currentTabId = tab.id;
		currentMsg = message;
		setIcon(MAILZ_STATUS_LOADING);			
		EcoMailzSettings.loadSettings().then(() => {
		
			if(ecoMailzConnectionSettings.url === "" || ecoMailzConnectionSettings.url === null)
			{
				setIcon(MAILZ_STATUS_NOTCONFIGURED);
			}
			else
			{
				EcoMailzConnectionHandler.checkConnection(ecoMailzConnectionSettings, connectionSuccessCallback, errorCallback);	
			}			
		});
	//}
	//else
	//{
	//	browser.messageDisplayAction.disable(tab.id);
	//}
});

function setIcon(mStatus)
{
	currentStatus = mStatus;
	
	var icon = null;
	var title = null;
	var popup = null;
	var click = null;
	
	if(mStatus === MAILZ_STATUS_LOADING)
	{
		icon = MAILZ_LOGO_LOADING;
		title = EcoMailzHelper.getGlobalProperty(BUTTON_TEXT_LOADING);
	}	
	else if(mStatus === MAILZ_BUTTON_ID_ERROR || mStatus === MAILZ_STATUS_INVALID)
	{
		icon = MAILZ_LOGO_ERROR;
		title = EcoMailzHelper.getGlobalProperty(ERROR_ID_NO_CONNECTION_OR_ERROR);
	}	
	else if(mStatus === MAILZ_STATUS_NOTCONFIGURED)
	{
		icon = MAILZ_LOGO_NOTCONFIGURED;
		title = EcoMailzHelper.getGlobalProperty(ERROR_ID_NOT_CONFIGURED);
	}
	else if(mStatus === MAILZ_STATUS_ARCHIVED)
	{
		click = true;
		icon = MAILZ_LOGO_GREEN;
		title = EcoMailzHelper.getGlobalProperty(BUTTON_TEXT_ARCHIVED);
	}
	else if(mStatus === MAILZ_STATUS_NOTINMAILZ)
	{
		icon = MAILZ_LOGO_WHITE;
		title = EcoMailzHelper.getGlobalProperty(BUTTON_TEXT_NOT_IN_MAILZ);
	}
	else if(mStatus === MAILZ_STATUS_TOARCHIVE)
	{
		icon = MAILZ_LOGO_ORANGE;
		title = EcoMailzHelper.getGlobalProperty(BUTTON_TEXT_TO_ARCHIVE);
		popup = MAILZ_POPUP_TOARCHIVE;
	}	
	else if(mStatus === MAILZ_STATUS_TODELETE)
	{
		icon = MAILZ_LOGO_RED;
		title = EcoMailzHelper.getGlobalProperty(BUTTON_TEXT_MARKED_FOR_DELETE);
		popup = MAILZ_POPUP_MARKEDFORDELETE;
	}	
	else if(mStatus === MAILZ_STATUS_DELETED)
	{
		icon = MAILZ_LOGO_BLACK;
		title = EcoMailzHelper.getGlobalProperty(BUTTON_TEXT_DELETED);
	}
	else if(mStatus === MAILZ_STATUS_DELETIONREQUEST)
	{
		icon = MAILZ_LOGO_GREY;
		title = EcoMailzHelper.getGlobalProperty(BUTTON_TEXT_DELETION_REQUEST);
	}
	else if(mStatus === MAILZ_STATUS_EMG)
	{
		icon = MAILZ_LOGO_EMG;
		title = EcoMailzHelper.getGlobalProperty(BUTTON_TEXT_EMG);
	}	
	
	if(icon !== null)
	{
    browser.messageDisplayAction.setIcon({
      path: {
        64: icon,
      },
      tabId: currentTabId,
    });
	}	
	else
	{
    browser.messageDisplayAction.setIcon({
      path: null,
      tabId: currentTabId,
    });
	}
	
	browser.messageDisplayAction.setTitle({ tabId: currentTabId, title: title });
	browser.messageDisplayAction.setPopup({ tabId: currentTabId, popup: popup });
	
	if(click)
	{
		browser.messageDisplayAction.onClicked.addListener(onIconClicked);
	}
	else
	{
		browser.messageDisplayAction.onClicked.removeListener(onIconClicked);
	}		
}

function onIconClicked()
{
	if(currentStatus === MAILZ_STATUS_ARCHIVED)
	{
		if(emgCount === EMG_COUNT)
		{
			setIcon(MAILZ_STATUS_EMG);
		}
		else if(emgCount <= EMG_COUNT)
		{
			emgCount++;
		}			
	}
}


async function connectionSuccessCallback(response, type, xml, text)
{
	var fullMsg = await browser.messages.getFull(currentMsg.id);
	var msgId = getHeader(MAIL_HEADER_MESSAGE_ID, fullMsg);
	var deliveredTo = getHeader(MAIL_HEADER_DELIVERED_TO, fullMsg);
	
	if(deliveredTo === null)
	{
		deliveredTo = "";
	}		
	
	if(msgId === null || msgId === "")
	{
		msgId = await generateNewMessageId(fullMsg);
	}		
	
	var encryptedMsgId = await EcoMailzEncryptionHandler.performMessageIdEncryption(msgId + deliveredTo);
	var encryptedDeletedMsgId = await EcoMailzEncryptionHandler.performMessageIdEncryption(MESSAGE_ID_PREFIX_DELETED + msgId);

	var encMsgIdHex = EcoMailzHelper.convertBufferToHexString(encryptedMsgId);
	var encDelMsgIdHex = EcoMailzHelper.convertBufferToHexString(encryptedDeletedMsgId);
	
	EcoMailzRestService.searchMail(ecoMailzConnectionSettings, " (" + SEARCH_MAIL_CHECKSUM + encDelMsgIdHex + " OR " + SEARCH_MAIL_CHECKSUM + encMsgIdHex + ") ", searchMailSuccessCallback, errorCallback);
}

function searchMailSuccessCallback(response, type, xml, text)
{
    var result = EcoMailzRestService.getEcoMailZResult(text);
    
    if(result.resultMails.size === 1)
    {
        var mail = null;
        
        for (var [key, value] of result.resultMails) 
        {
            mail = value;
        } 
	
		currentlySelectedMail = mail;
         
        var isDeletionRequest = false;
        var status = "";
                
        if(mail.optionalIndexerStatus !== null && mail.optionalIndexerStatus.toString() == INDEX_OPTIONAL_STATUS_DELETION_REQUEST)
        {
            isDeletionRequest = true;
        }
        
        if(mail.status !== null)
        {
            status = mail.status.toString();
        }
        
        if(isDeletionRequest)
        {
            ecoMailzConnection.currentStatus = MAILZ_STATUS_DELETIONREQUEST;
        }
        else if(MAILZ_INTERNAL_STATUS_ARRAY_ARCHIVED.includes(status))
        {
             ecoMailzConnection.currentStatus = MAILZ_STATUS_ARCHIVED;
        }
        else if(MAILZ_INTERNAL_STATUS_ARRAY_TOARCHIVE.includes(status))
        {
            ecoMailzConnection.currentStatus = MAILZ_STATUS_TOARCHIVE;
        }
        else if(MAILZ_INTERNAL_STATUS_ARRAY_TODELETE.includes(status))
        {
            ecoMailzConnection.currentStatus = MAILZ_STATUS_TODELETE;
        }
        else if(MAILZ_INTERNAL_STATUS_ARRAY_DELETED.includes(status))
        {
            ecoMailzConnection.currentStatus = MAILZ_STATUS_DELETED;
        }
        else
        {
            ecoMailzConnection.currentStatus = MAILZ_STATUS_ERROR;
        }
        
    }
    else if(result.resultMails.size === 0)
    {
        ecoMailzConnection.currentStatus = MAILZ_STATUS_NOTINMAILZ;
    }
    else
    {
		 ecoMailzConnection.currentStatus = MAILZ_STATUS_ERROR;
    }
    
	setIcon(ecoMailzConnection.currentStatus);			
       
}

async function generateNewMessageId(fullMsg, deliveredTo)
{
	var data = await browser.messages.getRaw(currentMsg.id);
    var tmpEncr = await EcoMailzEncryptionHandler.performMessageIdEncryption(data);
	var newMessageId = EcoMailzHelper.arrayToBase64String(tmpEncr) + MAILZ_GENERATE_MESSAGEID_POSTFIX;

	return newMessageId;
}

function str2ab(str) {
  var buf = new ArrayBuffer(str.length*2); // 2 bytes for each char
  var bufView = new Uint16Array(buf);
  for (var i=0, strLen=str.length; i<strLen; i++) {
    bufView[i] = str.charCodeAt(i);
  }
  return buf;
}

function getHeader(headerName, fullMsg)
{
	var header = fullMsg.headers[headerName];
	
	if(header && header.length > 0)
	{
		header = header[0];
	}
	else
	{
		header = null;
	}

	return header;	
}

async function errorCallback(errorMsg)
{
	setIcon(MAILZ_STATUS_ERROR);			
}

