window.browser = window.browser.extension.getBackgroundPage().browser;

async function init() {
  translate();
  resetMailzInfo();
  await EcoMailzSettings.loadSettings();
  loadSettingsIntoOptions();
  updateUrlString();
  window.document.getElementById(SETTINGS_SUBMIT_BUTTON).style.display = "none";
}

function getUrlString()
{
	var url;
	
    if(window.document.getElementById(SETTINGS_FILE_SSL).checked)
    {
        url = SETTINGS_SSL;
    }
    else
    {
        url = SETTINGS_NO_SSL;
    }
    
    url += window.document.getElementById(SETTINGS_FILE_SERVER).value;
    url += ":";
    url += window.document.getElementById(SETTINGS_FILE_PORT).value;
    url += window.document.getElementById(SETTINGS_FILE_APIPATH).value;

	return url;	
}

function updateUrlString()
{
	var url = getUrlString();

	window.document.getElementById(SETTINGS_FILE_URL).value = url;
}

function setSettingsIntoObject()
{
	ecoMailzConnectionSettings.server = document.getElementById(SETTINGS_FILE_SERVER).value;
    ecoMailzConnectionSettings.port = document.getElementById(SETTINGS_FILE_PORT).value;
    ecoMailzConnectionSettings.apiPath = document.getElementById(SETTINGS_FILE_APIPATH).value;
    ecoMailzConnectionSettings.ssl = document.getElementById(SETTINGS_FILE_SSL).checked;
    ecoMailzConnectionSettings.username = document.getElementById(SETTINGS_FILE_USERNAME).value;
    ecoMailzConnectionSettings.password = document.getElementById(SETTINGS_FILE_PASSWORD).value;
    ecoMailzConnectionSettings.url = document.getElementById(SETTINGS_FILE_URL).value;
}

function loadSettingsIntoOptions()
{
	window.document.getElementById(SETTINGS_FILE_SERVER).value = ecoMailzConnectionSettings.server;
	window.document.getElementById(SETTINGS_FILE_PORT).value = ecoMailzConnectionSettings.port;
	window.document.getElementById(SETTINGS_FILE_APIPATH).value = ecoMailzConnectionSettings.apiPath;
	window.document.getElementById(SETTINGS_FILE_SSL).checked = ecoMailzConnectionSettings.ssl;
	window.document.getElementById(SETTINGS_FILE_USERNAME).value = ecoMailzConnectionSettings.username;
	window.document.getElementById(SETTINGS_FILE_PASSWORD).value = ecoMailzConnectionSettings.password;
}

init();

function updateMailzInfo(text, isError)
{
	var mailzInfo = window.document.getElementById(SETTINGS_MAILZ_INFO);
	mailzInfo.innerHTML = text;
	
	if(isError)
	{
		mailzInfo.classList.remove("mailzSucess");
		mailzInfo.classList.add("mailzError");
	}		
	else
	{
		mailzInfo.classList.remove("mailzError");
		mailzInfo.classList.add("mailzSucess");
	}		
}

function resetMailzInfo()
{
	window.document.getElementById(SETTINGS_MAILZ_INFO).innerHTML = "";
}

function testClicked()
{
	resetMailzInfo();
	if(checkFields())
	{
		setSettingsIntoObject();
		EcoMailzConnectionHandler.checkConnection(ecoMailzConnectionSettings, connectionSuccessCallback, connectionErrorCallback);
	}
	else
	{
		toggleSaveButton(false);
		updateMailzInfo(EcoMailzHelper.getGlobalProperty(ERROR_ID_SETTINGS_FIELD_MANDATORY), true);	
	}	
}

function connectionSuccessCallback(response, type, xml, text)
{
	updateMailzInfo(EcoMailzHelper.getGlobalProperty(CONNECTION_SUCCESSFUL_PROPERTY, false));
	toggleSaveButton(true);
}

function connectionErrorCallback(errorMsg)
{
	toggleSaveButton(false);
	updateMailzInfo(EcoMailzHelper.getGlobalProperty(CONNECTION_FAILED_PROPERTY) + " " + errorMsg, true);
}

async function submitClicked(e)
{
	resetMailzInfo();
	if(checkFields())
	{
		setSettingsIntoObject();
		ecoMailzConnectionSettings = await EcoMailzEncryptionHandler.setEncryptedPassword(ecoMailzConnectionSettings);
		await EcoMailzSettings.saveSettings();
		updateMailzInfo(EcoMailzHelper.getGlobalProperty(SETTINGS_SAVED), false);
	}
	else
	{
		updateMailzInfo(EcoMailzHelper.getGlobalProperty(ERROR_ID_SETTINGS_FIELD_MANDATORY), true);	
	}		
}

function checkFields()
{
	return checkField(window.document.getElementById(SETTINGS_FILE_SERVER)) &&
	checkField(window.document.getElementById(SETTINGS_FILE_PORT)) &&
	checkField(window.document.getElementById(SETTINGS_FILE_APIPATH)) &&
	checkField(window.document.getElementById(SETTINGS_FILE_USERNAME)) &&
	checkField(window.document.getElementById(SETTINGS_FILE_PASSWORD));
}

function checkField(field)
{
	var isFilled = false;
	
	if(field.value && field.value !== "")
	{
		isFilled = true;
	}
	
	return isFilled;
}

function toggleSaveButton(enable)
{
	var saveButton = window.document.getElementById(SETTINGS_SUBMIT_BUTTON);
	
	if(window.getComputedStyle(saveButton).display === "none" && enable)
	{
		window.document.getElementById(SETTINGS_SUBMIT_BUTTON).style.display = "inline-block";
	}
	else if(window.getComputedStyle(saveButton).display === "inline-block" && !enable)
    {
		window.document.getElementById(SETTINGS_SUBMIT_BUTTON).style.display = "none";
	}
}

function advancedClicked()
{
	var apiPath = window.document.getElementById(SETTINGS_API_PATH_CONTAINER);
	var theImg = window.document.getElementById("advancedButtonToggleImg");
	
	if(window.getComputedStyle(apiPath).display === "none")
	{
		window.document.getElementById(SETTINGS_API_PATH_CONTAINER).style.display = "block";
		theImg.src = SETTINGS_IMG_SOURCE_EXPAND_LESS;
	}
	else if(window.getComputedStyle(apiPath).display === "block")
    {
		window.document.getElementById(SETTINGS_API_PATH_CONTAINER).style.display = "none";
		theImg.src = SETTINGS_IMG_SOURCE_EXPAND_MORE;
	}		
}

function updateUrlStringAndDisableSave()
{
	resetMailzInfo();
	updateUrlString();
	toggleSaveButton(false);
}

window.document.getElementById(SETTINGS_FILE_SERVER).addEventListener('keyup', updateUrlStringAndDisableSave);
window.document.getElementById(SETTINGS_FILE_PORT).addEventListener('keyup', updateUrlStringAndDisableSave);
window.document.getElementById(SETTINGS_FILE_PORT).addEventListener('click', updateUrlStringAndDisableSave);
window.document.getElementById(SETTINGS_FILE_SSL).addEventListener('click', updateUrlStringAndDisableSave);
window.document.getElementById(SETTINGS_FILE_APIPATH).addEventListener('keyup', updateUrlStringAndDisableSave);
window.document.getElementById(SETTINGS_FILE_USERNAME).addEventListener('keyup', updateUrlStringAndDisableSave);
window.document.getElementById(SETTINGS_FILE_PASSWORD).addEventListener('keyup', updateUrlStringAndDisableSave);


window.document.getElementById('advancedButtonContainer').addEventListener('click', advancedClicked);
window.document.getElementById('mailzTestButton').addEventListener('click', testClicked);
window.document.getElementById(SETTINGS_SUBMIT_BUTTON).addEventListener('click', submitClicked);
