var EcoMailzEncryptionHandler = {
getHash(value)
{
    var hash = 0;

    if (value.length === 0) return hash;

    for (var i = 0; i < value.length; i++) {
        char = value.charCodeAt(i);
        hash = ((hash<<5)-hash)+char;
        hash = hash & hash; // Convert to 32bit integer
    }
    
    return hash;
},

async performMessageIdEncryption(str) {
  // We transform the string into an arraybuffer.
  var buffer = new TextEncoder(DEFAULT_ENCODING).encode(str);    
  return await window.crypto.subtle.digest(ENCRYPTION_MESSAGE_ID, buffer);
},

// only call from asynchronous function
getSharedSecret:async function(plainString)
{
    var encoder = new TextEncoder(DEFAULT_ENCODING);
            
    var ptUtf8 = encoder.encode(plainString);

    var sharedSecretArr = await window.crypto.subtle.digest(ENCRYPTION_HASH_FUNCTION, ptUtf8); 
    return EcoMailzHelper.arrayToBase64String(sharedSecretArr);
},

// only call from asynchronous function
getDecryptedString:async function(sharedSecret, encryptedString)
{
    var sharedSecretArr = EcoMailzHelper.base64StringToArray(sharedSecret);
    var decoder = new TextDecoder(DEFAULT_ENCODING);
    
    var ivDecrypt = EcoMailzHelper.asciiToUint8Array(ENCRYPTION_IV_STRING);
    var algDecrypt = { name: ENCRYPTION_ALGORITH, iv: ivDecrypt };

    var keyDecrypt = await window.crypto.subtle.importKey(ENCRYPTION_KEY_FORMAT, sharedSecretArr, algDecrypt, false, ['decrypt']);
    
    var ctBuffer = EcoMailzHelper.base64StringToArray(encryptedString);
    
    var ptBuffer = await window.crypto.subtle.decrypt(algDecrypt, keyDecrypt, ctBuffer);
    
    return decoder.decode(ptBuffer);
},

// only call from asynchronous function
getEncryptedString:async function(sharedSecret, plainString)
{
    var encoder = new TextEncoder(DEFAULT_ENCODING);
    var sharedSecretArr = EcoMailzHelper.base64StringToArray(sharedSecret);
    
    var ivEncrypt = EcoMailzHelper.asciiToUint8Array(ENCRYPTION_IV_STRING);
    var algEncrypt = { name: ENCRYPTION_ALGORITH, iv: ivEncrypt };

    var keyEncrypt = await window.crypto.subtle.importKey(ENCRYPTION_KEY_FORMAT, sharedSecretArr, algEncrypt, false, ['encrypt']);

    var ctBuffer = await window.crypto.subtle.encrypt(algEncrypt, keyEncrypt, encoder.encode(plainString));

    return EcoMailzHelper.arrayToBase64String(ctBuffer);
},

getEncodedUserdata:function(ecoMailzConnectionSettings)
{
    var encoder = new TextEncoder(DEFAULT_ENCODING);
    
    var encodedArray = encoder.encode(ecoMailzConnectionSettings.username + ":" + ecoMailzConnectionSettings.password);
    
    return EcoMailzHelper.arrayToBase64String(encodedArray);
},

setEncryptedPassword:async function(ecoMailzConnectionSettings)
{
    var password = ecoMailzConnectionSettings.password;
    var sharedSecret = await EcoMailzEncryptionHandler.getSharedSecret(ecoMailzConnectionSettings.password);
    
    ecoMailzConnectionSettings.sharedSecret = sharedSecret;
    
    var encryptedPassword = await EcoMailzEncryptionHandler.getEncryptedString(sharedSecret, password);
    
    ecoMailzConnectionSettings.encryptedPassword = encryptedPassword;
	
	return ecoMailzConnectionSettings;
}
};