var ecoMailzConnection = {
    connected:false,
    lastHash:"",
    currentStatus:""
};

var EcoMailzRestService = {
connect:function(ecoMailzConnectionSettings, successCallback, errorCallback)
{      
    return EcoMailzRestService.connectRequest(ecoMailzConnectionSettings, DEFAULT_REQUEST_GET_TIMEOUT, successCallback, errorCallback);
},

disconnect:function(url, successCallback, errorCallback)
{
    return EcoMailzRestService.disconnectRequest(url, DEFAULT_REQUEST_GET_TIMEOUT, successCallback, errorCallback);
},

connectRequest:function(ecoMailzConnectionSettings, timeout, successCallback, errorCallback)
{
    var request = new XMLHttpRequest();
        
    request.timeout = timeout;
	request.withCredentials = true;
    request.onreadystatechange = function() {
        if (this.readyState === 4) 
        {
            if(this.status === 200)
            {
                var successString = CONNECTION_SUCCESS_MESSAGE_PREFIX + ecoMailzConnectionSettings.username + CONNECTION_SUCCESS_MESSAGE_POSTFIX;
                
                if(request.responseText === successString)
                {
                    EcoMailzRestService.setConnected(true);
                
                    if(successCallback !== null)
                    {
                        successCallback(request.response, request.responseType, request.responseXML, request.responseText);                
                    }
                }
                else
                {
                    EcoMailzRestService.setConnected(false); 
                    
                    errorMsg = EcoMailzHelper.getGlobalProperty(ERROR_ID_URL_INVALID);
                    
                    if(errorCallback !== null)
                    {
                        errorCallback(errorMsg);
                    }
                }
            }        
            else
            {
                EcoMailzRestService.setConnected(false);
                var errorMsg;
                
                if(this.status === 0)
                {
                   errorMsg = EcoMailzHelper.getGlobalProperty(ERROR_ID_URL_INVALID);
                }
                else
                {
                    errorMsg = this.statusText;
                }
                
                if(errorCallback !== null)
                {
                    errorCallback(errorMsg);
                }
            }     
        }
    }
    
    try
    {
        request.open(REQUEST_TYPE_GET, EcoMailzRestService.getUrl(ecoMailzConnectionSettings.url) + REQUEST_GET_CONNECT, true);
        
        request = EcoMailzRestService.setRequestHeaderInRequest(request, EcoMailzRestService.getAuthorizationHeaderArray(ecoMailzConnectionSettings));
        request.send(null);
    }
    catch(error)
    {
        EcoMailzRestService.setConnected(false);
   
        if(errorCallback !== null)
        {
            errorCallback(error.message);
        }
     
        return;
    }
},

disconnectRequest:function(url, timeout, successCallback, errorCallback)
{
    var request = new XMLHttpRequest();
        
    request.timeout = timeout;
    request.withCredentials = true;
    request.onreadystatechange = function() {
        if (this.readyState === 4) 
        {
            if(this.status === 200)
            {
                EcoMailzRestService.setConnected(false);
                        
                if(successCallback !== null)
                {
                    successCallback(request.response, request.responseType, request.responseXML, request.responseText);                
                }
            }        
            else
            {       
                EcoMailzRestService.setConnected(false);

                var errorMsg;
                
                if(this.status === 0)
                {
                   errorMsg = EcoMailzHelper.getGlobalProperty(ERROR_ID_REST_GENERALERROR) + " " + REQUEST_GET_DISCONNECT;
                }
                else
                {
                    errorMsg = this.statusText;
                }
                
                if(errorCallback !== null)
                {
                    errorCallback(errorMsg);
                }
            }     
        }
    }
    
    try
    {
        request.open(REQUEST_TYPE_GET, EcoMailzRestService.getUrl(url) + REQUEST_GET_DISCONNECT, true);
        
        request = EcoMailzRestService.setRequestHeaderInRequest(request, null);
        
        request.send(null);
    }
    catch(error)
    {        
        EcoMailzRestService.setConnected(false);

        if(errorCallback !== null)
        {
            errorCallback(error.message);
        }
     
        return;
    }
},

disconnectAndConnect:function(ecoMailzConnectionSettings, connectSuccessCallback, connectErrorCallback)
{
    var request = new XMLHttpRequest();
        
    request.timeout = DEFAULT_REQUEST_GET_TIMEOUT;
    request.withCredentials = true;
    request.onreadystatechange = function() {
        if (this.readyState === 4) 
        {
            if(this.status === 200)
            {
                EcoMailzRestService.connect(ecoMailzConnectionSettings, connectSuccessCallback, connectErrorCallback);
            }        
            else
            {
                EcoMailzRestService.setConnected(false);

                var errorMsg;
                
                if(this.status === 0)
                {
                    errorMsg = EcoMailzHelper.getGlobalProperty(ERROR_ID_URL_INVALID);
                }
                else
                {
                    errorMsg = this.statusText;
                }

                if(connectErrorCallback !== null)
                {
                    connectErrorCallback(errorMsg);
                }
            }     
        }
    }
    
    try
    {
        request.open(REQUEST_TYPE_GET, EcoMailzRestService.getUrl(ecoMailzConnectionSettings.url) + REQUEST_GET_DISCONNECT, true);
        
        request = EcoMailzRestService.setRequestHeaderInRequest(request, null);
        
        request.send(null);
    }
    catch(error)
    {             
        EcoMailzRestService.setConnected(false);

        if(connectErrorCallback !== null)
        {
            connectErrorCallback(error.message);
        }
        
        return;
    }
},

searchMail:function(ecoMailzConnectionSettings, query, successCallback, errorCallback)
{
    var searchObject = new EcoMailzSearchTypes(query);
    
    var postData = EcoMailzRestService.getDataForPostRequest(searchObject);
    
    var requestHeaderArr = [];
    requestHeaderArr[0] = EcoMailzRestService.getAuthorizationHeader(ecoMailzConnectionSettings);
    
    requestHeaderArr[1] = new EcoMailzRequestHeader(POST_REQUEST_HEADER_CONTENT_TYPE, POST_REQUEST_HEADER_CONTENT_TYPE_DEFAULT_VALUE);
    //requestHeaderArr[2] = new EcoMailzRequestHeader(POST_REQUEST_HEADER_CONTENT_LENGTH, postData.length);

    return EcoMailzRestService.searchMailRequest(ecoMailzConnectionSettings, requestHeaderArr, DEFAULT_REQUEST_GET_TIMEOUT, postData, successCallback, errorCallback);
},

searchMailRequest:function(ecoMailzConnectionSettings, requestHeaderArray, timeout, params, successCallback, errorCallback)
{
    var request = new XMLHttpRequest();
        
    request.timeout = timeout;
    request.withCredentials = true;
    request.onreadystatechange = function() {
        if (this.readyState === 4) 
        {
            if(this.status === 200)
            {    
                if(successCallback !== null)
                {
                    successCallback(request.response, request.responseType, request.responseXML, request.responseText);                
                }
            }        
            else
            {
                var errorMsg;
                
                if(this.status === 0)
                {
                   errorMsg = EcoMailzHelper.getGlobalProperty(ERROR_ID_REST_GENERALERROR) + " " + REQUEST_POST_SEARCHMAIL;
                }
                else
                {
                    errorMsg = this.statusText;
                }
                
                if(errorCallback !== null)
                {
                    errorCallback(errorMsg);
                }
            }     
        }
    }
    
    try
    {
        request.open(REQUEST_TYPE_POST, EcoMailzRestService.getUrl(ecoMailzConnectionSettings.url) + REQUEST_POST_SEARCHMAIL, true);
        
        request = EcoMailzRestService.setRequestHeaderInRequest(request, requestHeaderArray);
        
        request.send(params);
    }
    catch(error)
    {   
        if(errorCallback !== null)
        {
            errorCallback(error.message);
        }
     
        return;
    }
},

markDeletion:function(ecoMailzConnectionSettings, mailDao, successCallback, errorCallback)
{
    var requestHeaderArr = EcoMailzRestService.getAuthorizationHeaderArray(ecoMailzConnectionSettings);
    var postData = EcoMailzRestService.getDataForPostRequest(mailDao);
    
    requestHeaderArr[1] = new EcoMailzRequestHeader(POST_REQUEST_HEADER_CONTENT_TYPE, POST_REQUEST_HEADER_CONTENT_TYPE_DEFAULT_VALUE);
    //requestHeaderArr[2] = new EcoMailzRequestHeader(POST_REQUEST_HEADER_CONTENT_LENGTH, postData.length);

    return EcoMailzRestService.mailDaoRequest(ecoMailzConnectionSettings, requestHeaderArr, REQUEST_POST_MARKDELETION, DEFAULT_REQUEST_GET_TIMEOUT, postData, MAILZ_STATUS_TODELETE, successCallback, errorCallback);
},

undoDeletion:function(ecoMailzConnectionSettings, mailDao, successCallback, errorCallback)
{
    var requestHeaderArr = EcoMailzRestService.getAuthorizationHeaderArray(ecoMailzConnectionSettings);
    var postData = EcoMailzRestService.getDataForPostRequest(mailDao);
    
    requestHeaderArr[1] = new EcoMailzRequestHeader(POST_REQUEST_HEADER_CONTENT_TYPE, POST_REQUEST_HEADER_CONTENT_TYPE_DEFAULT_VALUE);
    //requestHeaderArr[2] = new EcoMailzRequestHeader(POST_REQUEST_HEADER_CONTENT_LENGTH, postData.length);

    return EcoMailzRestService.mailDaoRequest(ecoMailzConnectionSettings, requestHeaderArr, REQUEST_POST_UNDODELETION, DEFAULT_REQUEST_GET_TIMEOUT, postData, MAILZ_STATUS_TOARCHIVE, successCallback, errorCallback);
},

skipDeadline:function(ecoMailzConnectionSettings, mailDao, successCallback, errorCallback)
{
    var requestHeaderArr = EcoMailzRestService.getAuthorizationHeaderArray(ecoMailzConnectionSettings);
    var postData = EcoMailzRestService.getDataForPostRequest(mailDao);
    
    requestHeaderArr[1] = new EcoMailzRequestHeader(POST_REQUEST_HEADER_CONTENT_TYPE, POST_REQUEST_HEADER_CONTENT_TYPE_DEFAULT_VALUE);
    //requestHeaderArr[2] = new EcoMailzRequestHeader(POST_REQUEST_HEADER_CONTENT_LENGTH, postData.length);

    return EcoMailzRestService.mailDaoRequest(ecoMailzConnectionSettings, requestHeaderArr, REQUEST_POST_SKIPDEADLINE, DEFAULT_REQUEST_GET_TIMEOUT, postData, MAILZ_STATUS_ARCHIVED, successCallback, errorCallback);
},

undoSkipDeadline:function(ecoMailzConnectionSettings, mailDao, successCallback, errorCallback)
{
    var requestHeaderArr = EcoMailzRestService.getAuthorizationHeaderArray(ecoMailzConnectionSettings);
    var postData = EcoMailzRestService.getDataForPostRequest(mailDao);
    
    requestHeaderArr[1] = new EcoMailzRequestHeader(POST_REQUEST_HEADER_CONTENT_TYPE, POST_REQUEST_HEADER_CONTENT_TYPE_DEFAULT_VALUE);
    //requestHeaderArr[2] = new EcoMailzRequestHeader(POST_REQUEST_HEADER_CONTENT_LENGTH, postData.length);

    return EcoMailzRestService.mailDaoRequest(ecoMailzConnectionSettings, requestHeaderArr, REQUEST_POST_UNDOSKIPDEADLINE, DEFAULT_REQUEST_GET_TIMEOUT, postData, MAILZ_STATUS_TOARCHIVE, successCallback, errorCallback);
},


getDataForPostRequest:function(object)
{
    return JSON.stringify(object);
},

mailDaoRequest:function(ecoMailzConnectionSettings, requestHeaderArray, service, timeout, params, statusToSetAfterSuccess, successCallback, errorCallback)
{
    var request = new XMLHttpRequest();
        
    request.timeout = timeout;
    request.withCredentials = true;
    request.onreadystatechange = function() {
        if (this.readyState === 4) 
        {
            if(this.status === 200)
            {    
                if(request.responseText === BOOLEAN_REQUEST_SUCCESS)
                {
                    ecoMailzConnection.currentStatus = statusToSetAfterSuccess;
                }
                else
                {
                   errorMsg = EcoMailzHelper.getGlobalProperty(ERROR_ID_REST_GENERALERROR) + " " + service;
                }
                
                if(successCallback !== null)
                {
                    successCallback(request.response, request.responseType, request.responseXML, request.responseText);                
                }
            }        
            else
            {
                var errorMsg;
                
                if(this.status === 0)
                {
                   errorMsg = EcoMailzHelper.getGlobalProperty(ERROR_ID_REST_GENERALERROR) + " " + service;
                }
                else
                {
                    errorMsg = this.statusText;
                }
                
                if(errorCallback !== null)
                {
                    errorCallback(errorMsg);
                }
            }     
        }
    }
    
    try
    {
        request.open(REQUEST_TYPE_POST, EcoMailzRestService.getUrl(ecoMailzConnectionSettings.url) + service, true);
        
        request = EcoMailzRestService.setRequestHeaderInRequest(request, requestHeaderArray);
        
        request.send(params);
    }
    catch(error)
    { 
        if(errorCallback !== null)
        {
            errorCallback(error.message);
        }
     
        return;
    }
},

getUrl:function(url)
{
    if(!url.endsWith("/"))
    {
        url = url+"/";
    }
    
    return url;
},

setRequestHeaderInRequest:function(request, requestHeaderArray)
{
    if(requestHeaderArray !== null && requestHeaderArray.length !== null)
    {
        for(var i = 0; i < requestHeaderArray.length; i++)
        {
            var header = requestHeaderArray[i];
            request.setRequestHeader(header.name, header.value);
        }
    }
    
    return request;
},

getAuthorizationHeaderArray:function(ecoMailzConnectionSettings)
{
    var requestHeaderArr = [];
    requestHeaderArr[0] = EcoMailzRestService.getAuthorizationHeader(ecoMailzConnectionSettings);

    return requestHeaderArr;
},

getAuthorizationHeader:function(ecoMailzConnectionSettings)
{
    return new EcoMailzRequestHeader(CONNECT_REQUEST_HEADER_AUTHORIZATION, CONNECT_REQUEST_USERDATA_PREFIX + " " + EcoMailzEncryptionHandler.getEncodedUserdata(ecoMailzConnectionSettings));
},

getEcoMailZResult:function(responseText)
{
    var jsonObject = JSON.parse(responseText);
    
    var resultMailsMap = EcoMailzHelper.convertObjectToMap(jsonObject.resultMails);
    
    return new EcoMailZResult(resultMailsMap, jsonObject.lastScore, jsonObject.totalHits);
},

setConnected:function(connected)
{
    if(!ecoMailzConnection.connected === connected)
    {
        ecoMailzConnection.connected = connected;        
    }
}
}